#!/bin/sh

source /usr/local/bin/fw_common.sh

FW_LOG_CTL=/usr/local/bin/fwlogctl
FW_BOOT_LOG=/tmp/boot.log

FW_LOG_CATEGORY_SYSTEM=10
FW_LOG_TYPE_SYSTEM_BOOT=1

function check_boot_log() {
    if [ ! -f "$FW_BOOT_LOG" ]; then
        $JOURNALCTL -k --no-pager > "$FW_BOOT_LOG"
    fi
}

function add_one_line() {
    local line="$1"
    local category=$2
    local type=$3
    if [ -z $category ]; then
        category=$FW_LOG_CATEGORY_SYSTEM
    fi
    if [ -z $type ]; then
        type=$FW_LOG_TYPE_SYSTEM_BOOT
    fi
    $FW_LOG_CTL add -c $category -t $type -d "$line"  -u "SYSTEM"
    # $ECHO "$FW_LOG_CTL add -c $category -t $type -d \"$line\""
}

function add_multi_lines() {
    local lines="$1"
    local category=$2
    local type=$3
    $ECHO "$lines" | while IFS= read -r l; do
        add_one_line "$l" $category $type
    done
    $ECHO
}

function check_cpu() {
    local patterns=("All CPU(s) started" "Testing write buffer coherency: ok")
    for p in "${patterns[@]}"; do
        local lines=`$CAT $FW_BOOT_LOG | $GREP "$p"`
        local existed=`$ECHO "$lines" | $WC -l`
        if [ $existed -eq 0 ]; then
            $ECHO "CPU: NG"
            return -1
        fi
        $ECHO "$lines"
    done
    $ECHO "CPU: OK"
    return 0
}

function check_memory() {
    local lines=`$CAT $FW_BOOT_LOG | $GREP "Memory:"`
    local count=`$ECHO "$lines" | $WC -l`
    if [ "$count" -lt 2 ]; then
        $ECHO "Memory: NG"
        return -1
    fi
    $ECHO "$lines"
    $ECHO "Memory: OK"
    return 0
}

function check_flash() {
    local lines=`$CAT $FW_BOOT_LOG | $GREP "NAND device:"`
    local count=`$ECHO "$lines" | $WC -l`
    if [ "$count" -lt 1 ]; then
        $ECHO "NAND: NG"
        return -1
    fi
    $ECHO "$lines"
    $ECHO "NAND: OK"
    return 0
}

function check_network() {
    local lines=`/sbin/ifconfig | $GREP Link | $GREP -v lo`
    local count=`$ECHO $lines | $WC -l`
    if [ $count -eq 0 ]; then
        $ECHO "Network: NG"
        return -1
    fi
    $ECHO "$lines"
    $ECHO "Network: OK"
    return 0
}

function check_systemd_services() {
    local services="fw-init_sysinfo.service fw-vmf.service fw-rtspd.service fw-goahead.service"
    for s in $services; do
        local status=`/usr/bin/systemctl status "$s"`
        # $ECHO "$status" | $GREP "Loaded: loaded" | $GREP enabled
        is_enable=`$ECHO "$status" | $GREP "Loaded: loaded" | $GREP enabled | $WC -l`
        # $ECHO "is_enable: $is_enable"
        if [ $is_enable -eq 0 ]; then
            $ECHO "$s: NG"
            continue
        fi
        is_failed=`$ECHO "$status" | $GREP "Active: failed" | $WC -l`
        # $ECHO "is_failed: $is_failed"
        if [ $is_failed -ne 0 ]; then
            $ECHO "$s: NG"
            continue
        fi
        $ECHO "$s: OK"
    done
}

function check_hardware() {
    add_multi_lines "`check_cpu`"
    add_multi_lines "`check_memory`"
    add_multi_lines "`check_flash`"
    add_multi_lines "`check_network`"
}

function check_software() {
    add_multi_lines "`check_systemd_services`"
}

check_boot_log
check_hardware
check_software
add_one_line "System Booting Done!"
