#!/bin/sh

. /usr/local/bin/fw_common.sh

function get_system_build_time() {
	local sys_date=`$CAT /etc/system.info.base | $GREP SYS_DATE | $AWK -F= '{print $2}' | $SED 's|/|-|g'`
	local seconds_of_sys_date=`$DATE -d"$sys_date" +%s`
	local seconds_of_current=`$DATE +%s`
	local diff
	((diff = $seconds_of_sys_date - $seconds_of_current))
	$ECHO $diff
}

function create_public_key() {
	local cert_file=$1
	local pub_key_file=$2
	/usr/bin/openssl x509 -pubkey -in $cert_file > $pub_key_file~
	local end_line=`$GREP -n 'END PUBLIC KEY' $pub_key_file~ | $AWK -F':' '{print $1}'`
	$HEAD -"$end_line" $pub_key_file~ > $pub_key_file
	$RM $pub_key_file~
}

function init_web_cert_and_key() {
	local certs_dir=/etc/sysconfig/certs
	$MKDIR -p $certs_dir
	local fwkeygen="/opt/nss/bin/fwkeygen"
	if [ -x $fwkeygen ]; then
		local web_cert="$certs_dir/self.crt"
		local web_key="$certs_dir/self.key"
		local pub_key="$certs_dir/self.pub"
		if [ -f ${web_cert}.bin ] && [ -f ${web_key}.bin ] && [ -f ${pub_key} ]; then
			$ECHO "web cert/key: OK"
			$RM -f ${web_cert} ${web_key}
			return
		fi
		$RM -f ${web_cert}.bin ${web_key}.bin

		if [ -f ${web_cert} ] && [ -f ${web_key} ] && [ -f ${pub_key} ]; then
			$ECHO "web cert/key: OK"
			return
		fi
		local start_time=`get_system_build_time`
		${fwkeygen} -c ${web_cert} -k ${web_key} -y 10 -s ${start_time}
		create_public_key ${web_cert} ${pub_key}
		$ECHO "web cert/key initialization done"
	fi
}

init_web_cert_and_key
/bin/sync
$ECHO "web cert/sync done"