#!/bin/sh
# decrypt base64 encoded data, which has been encrypted with public key, with private key

. /usr/local/bin/fw_common.sh

OPENSSL=/usr/bin/openssl

function base64_decode_and_save() {
    local base64_encoded=$1
    local outfile=$2

    $OPENSSL enc -base64 -d <<< $base64_encoded > $outfile
}

function decrypt_with_private_key() {
    local in_file="$1"
    local private_key="$2"
    local tmp_key_file=`/bin/mktemp`
    $ECHO "$private_key" > $tmp_key_file
    $OPENSSL rsautl -decrypt -in $in_file -inkey $tmp_key_file
    $RM -f $tmp_key_file
}

if [ $# -lt 1 ]; then
    echo "Usage: `$BASENAME $0` encrypted_base64_str [private_key]"
    exit -1
fi

BASE64_STR=$1
PRIVATE_KEY="$2"
if [ -z "$PRIVATE_KEY" ]; then
    PRIVATE_KEY=`$CAT /etc/sysconfig/certs/self.key`
fi

TMP_ENC_FILE=`/bin/mktemp`

base64_decode_and_save $BASE64_STR $TMP_ENC_FILE
DECRYPTED=`decrypt_with_private_key $TMP_ENC_FILE "$PRIVATE_KEY"`

$RM -f $TMP_ENC_FILE
$ECHO $DECRYPTED
