#!/bin/sh

#########################################################################################################
# Initialize FlexWATCH System infomation
#########################################################################################################
source /usr/local/bin/p2p_common.sh

function check_p2p_info() {
	init_p2p
}

function check_hostpad_conf() {
	local wifi_type=`get_wifi_type`

	if [ `is_hostap_supported $wifi_type` -ne 1 ]; then
		$ECHO "not support hostap"
		return
	fi

	if [ -f $HOSTAPD_CONF ]; then
		saved_hostapd_ssid=`read_hostapd_conf_value $HOSTAPD_CONF_NAME_SSID`
		saved_hostapd_pwd=`read_hostapd_conf_value $HOSTAPD_CONF_NAME_PWD`
		if [ -n $saved_hostapd_ssid -a -n $saved_hostapd_pwd ]; then
			$ECHO "$HOSTAPD_CONF is OK."
			return
		fi
	fi

	local hostpad_ssid=`get_hostpad_ssid`
	local hostapd_password=`get_hostapd_password`
	init_hostapd_conf $hostpad_ssid $hostapd_password
}

function create_temp_env_dir() {
	$MKDIR -p $TMP_FW_ENV
}

function init_sensor_info() {
	FW_SENSOR=`$SPCP 19 2`
	$ECHO "sensor: $FW_SENSOR"
	FW_SENSOR_FILE=/var/log/fw_sensor_pncode
	FW_SENSOR_INTERLACE=/var/log/fw_sensor_interlace
	$ECHO "FW_SENSOR=$FW_SENSOR" > $FW_SENSOR_FILE
	$ECHO "FW_SENSOR_INTERLACE=0" > $FW_SENSOR_INTERLACE
	$ECHO "sensor info initialized"
}

function init_sys_info() { # /etc/motd, /etc/system.info
	if [ ! -f "/proc/driver/sy_basic_conf" ]; then
		$ECHO "[$0] Driver load failed."
	fi

	local fw_model=`$CAT /proc/driver/sy_basic_conf | $GREP SYF_STR0 | $SED 's/^SYF_STR0\="//g; s/"\;$//g'`
	local motd_file=/etc/motd
	local sys_info_file=/etc/system.info

	$CAT ${motd_file}.base > ${motd_file}
	$ECHO " MODEL          : $fw_model" >> ${motd_file}
	$ECHO " ------------------------------------------------" >> ${motd_file}
	$ECHO >> ${motd_file}

	$CAT ${sys_info_file}.base > ${sys_info_file}
	set_cfg_env_no_quot $sys_info_file "SYS_MODEL" $fw_model
}

function init_led() {
	local RGB_LED_CTL=/usr/local/bin/fw_rgb_led_ctl
	if [ -f $RGB_LED_CTL ]; then
		$RGB_LED_CTL -i
		if [ $? -ne 0 ]; then
			$ECHO "fail to init rbg led ctl"
			return
		fi
		$RGB_LED_CTL -c b
		$ECHO "rgb led ctl initialized"
	fi
}

function check_enc_key_data() {
	local enc_key_data_length=62
	local enc_key_data=`$SYFTST -r 8 | $AWK -F'=' '{print $2}' | $TR -d ' '`
	if [ ${#enc_key_data} -eq $enc_key_data_length ]; then
		return 0
	fi
	enc_key_data=`$CAT /dev/urandom | $TR -dc 'A-F0-9' | /usr/bin/fold -w $enc_key_data_length | $HEAD -n 1`
	$SYFTST -w 8 -s $enc_key_data
}

function check_security_option() {
	local security_debug=/root/.security_debug
	if [ -f $security_debug ]; then
		$ECHO "SECURITY DEBUG enabled!"
		return 0
	fi
	local services_to_remove="ftp.socket telnet.socket"
	for s in $services_to_remove; do
		local is_active=`$SYSTEMCTL status $s | $GREP "Active: active" | $WC -l`
		if [ $is_active -ne 0 ]; then
			$SYSTEMCTL stop $s
			$SYSTEMCTL disable $s
		fi
	done
	rm -f /sbin/pure-ftpd /usr/sbin/telnetd
}

# function for GPIO 102 with Input...
function export_gpio_102() {

GPIO_RTC=`$SPCP 37 1`
echo "RTC PNCODE = $GPIO_RTC."
if [ "$GPIO_RTC" != "G" ]; then
cd /sys/class/gpio
echo 102 > export
cd gpio102
echo in > direction
cd /sys/class/gpio/gpio105
echo in > direction
else
cd /sys/class/gpio
echo 102 > export
cd gpio102
echo out > direction
fi
}

IS_SECURITY_OPTION=`$SPCP 38 1`

create_temp_env_dir
init_sensor_info
init_sys_info
# Export GPIO 102 with Input...
export_gpio_102

if [ "$IS_SECURITY_OPTION" -ne 0 ]; then
	check_enc_key_data
	if [ "$IS_SECURITY_OPTION" -eq 1 ]; then 
		check_security_option
	fi
fi
check_p2p_info
check_hostpad_conf
#init_led

$ECHO
$ECHO "system info initialization done"
