#!/bin/sh

source /usr/local/bin/fw_common.sh

function set_wifi_env() {
	WIFI_TYPE_NONE="0"
	WIFI_TYPE_STATION="1"
	WIFI_TYPE_HOSTAPD="A"

	WLAN_MODE_CONF=/opt/nss/etc/wlan_mode.conf
	WLAN_MODE_STATION="STATION"
	WLAN_MODE_HOSTAP="HOSTAP"
	WLAN_MODE_DOWN="down"

	HOSTAPD_DIR=/etc/hostapd
	HOSTAPD_CONF=$HOSTAPD_DIR/hostapd.conf
	HOSTAPD_CONF_NAME_SSID=ssid2
	HOSTAPD_CONF_NAME_PWD=wpa_passphrase

	SCANNED_AP_LIST=$TMP_FW_ENV/scanned_ap_list
}

function get_wifi_type() {
	$SPCP 17 1
}

function is_wlan_supported() {
	local wifi_type=$1
	if [ -z $wifi_type ]; then
		wifi_type=`get_wifi_type`
	fi

	if [ "$wifi_type" == "$WIFI_TYPE_NONE" ]; then
		$ECHO "0"
	else
		$ECHO "1"
	fi
}

function is_hostap_supported() {
	local wifi_type=$1
	if [ -z $wifi_type ]; then
		wifi_type=`get_wifi_type`
	fi
	if [ "$wifi_type" == "$WIFI_TYPE_HOSTAPD" ]; then
		$ECHO "1"
	else
		$ECHO "0"
	fi
}

function get_wlan_mode() {
	if [ ! -f "$WLAN_MODE_CONF" ]; then
		$ECHO ""
		return
	fi
	$ECHO `cat $WLAN_MODE_CONF | awk -F'=' '{print $2}'`
}

function is_wlan_mode_hostap() {
	local wlan_mode=$1
	if [ -z $wlan_mode ]; then
		wlan_mode=`get_wlan_mode`
	fi
	if [ "$wlan_mode" == "$WLAN_MODE_HOSTAP" ]; then
		$ECHO "1"
	else
		$ECHO "0"
	fi
}

function set_wlan_mode() {
	local wlan_mode=$1
	$ECHO "WLAN_DEV_MODE=$wlan_mode" > $WLAN_MODE_CONF
}

function check_hostapd_conf_value() {
	local conf_name="$1"
	local conf_default_value="$2"
	local is_included=`$CAT ${HOSTAPD_CONF} | $GREP -cw "$conf_name"`
	if [ $is_included -eq 0 ]; then
		set_cfg_env_no_quot "${HOSTAPD_CONF}" "$conf_name" "$conf_default_value"
	fi
}

function read_hostapd_conf_value() {
	local conf_name="$1"
	$ECHO `$CAT $HOSTAPD_CONF | $GREP "$conf_name" | $AWK -F'-' '{print $2}'`
}

function init_hostapd_conf() {
	local conf_ssid="$1"
	local conf_password="$2"

	if [ -z "$conf_ssid" ]; then
		$ECHO "[$LINENO] invalid hostpad ssid"
		return -1
	fi
	if [ -z "$conf_password" ]; then
		$ECHO "[$LINENO] invalid hostpad password"
		return -1
	fi

	if [ ! -d $HOSTAPD_DIR ]; then
		$MKDIR -p $HOSTAPD_DIR
		$ECHO "$HOSTAPD_DIR created"
	fi
	if [ ! -f "$HOSTAPD_CONF" ]; then
		$TOUCH $HOSTAPD_CONF
		$ECHO "hostapd cont file created"
	fi

	set_cfg_env_no_quot $HOSTAPD_CONF $HOSTAPD_CONF_NAME_SSID "$conf_ssid"
	set_cfg_env_no_quot $HOSTAPD_CONF $HOSTAPD_CONF_NAME_PWD "$conf_password"
	set_cfg_env_no_quot $HOSTAPD_CONF utf8_ssid 1
	# other conf values would be initialized in init_hostapd.sh
}

function kill_wpa_supplicant() {
	/usr/bin/systemctl stop wpa_supplicant
	kill_proc wpa_supplicant
}

function down_bcmdhd_driver() {
	/sbin/modprobe -r bcmdhd
	if [[ $? -eq 0 ]]; then
		while [[ -d /sys/class/net/wlan0 ]]; do
			/bin/usleep 100000;
		done
	fi
	/bin/usleep 100000;
}

function up_bcmdhd_driver() {
	local opt_op_mode=$1
	$ECHO "/sbin/modprobe bcmdhd firmware_path=/lib/wifi/fw_bcmdhd.bin nvram_path=/lib/wifi/bcmdhd.cal iface_name=wlan0 $opt_op_mode"
	/sbin/modprobe bcmdhd firmware_path=/lib/wifi/fw_bcmdhd.bin nvram_path=/lib/wifi/bcmdhd.cal iface_name=wlan0 $opt_op_mode
	if [[ $? -eq 0 ]]; then
		while [[ ! -d /sys/class/net/wlan0 ]]; do
			/bin/usleep 100000;
		done
	fi
	/bin/usleep 500000;
}

function scan_ap_list() {
	/usr/sbin/iw wlan0 scan trigger > /dev/null
	/usr/bin/nmcli -f SSID-HEX,BSSID,CHAN,SIGNAL,SECURITY,FREQ dev wifi list > $SCANNED_AP_LIST
}

function get_scanned_ap_list() {
	local to_scan=$1
	if [ "$to_scan" == "1" ] || [ ! -f "$SCANNED_AP_LIST" ]; then
		scan_ap_list
	fi

	if [ ! -f "$SCANNED_AP_LIST" ]; then
		$ECHO "no scanned ap list"
		return -1
	fi

	$CAT "$SCANNED_AP_LIST"
}

function get_base_mac() {
	$SYFTST -r 6 | $AWK -F'=' '{print $2}' | $TR -d ' '
}

function increase_mac() {
	local base_mac=$1
	local offset_from_eth_mac=$2

	if [ -z $base_mac ]; then
		base_mac=`get_base_mac`
	fi
	if [ -z $offset_from_eth_mac ]; then
		offset_from_eth_mac=1
	fi

	local base_mac_as_hex=`$ECHO $base_mac | $TR -d ':'`
	local mac_as_decimal=`$ECHO $((0x$base_mac_as_hex))`
	((mac_as_decimal += $offset_from_eth_mac))
	local converted=`/usr/bin/printf "%012X" $mac_as_decimal | $SED 's/[0-9A-F][0-9A-F]/&:/g'`
	$ECHO ${converted%:}
}

set_wifi_env
